<?php

class WC_Ucc
{
    public function __construct()
    {
        add_action('register_activation_hook',      [$this, 'install']);
        add_action('admin_menu', function() {
            add_options_page('UCC plugin settings', 'UCC', 'manage_options', 'ucc', array($this, 'pluginOptions'));
        });

        add_action('woocommerce_init',                  [$this, 'hookWooCommerceInit'],     20);
        add_action('woocommerce_after_single_product',  [$this, 'hookAfterSingleProduct'],  20);
        add_action('woocommerce_thankyou',              [$this, 'hookOrderReceived'],       20);
        add_action('woocommerce_product_options_inventory_product_data', [$this, 'hookProductInventory'], 20);
        add_action('woocommerce_process_product_meta',  [$this, 'hookUpdateProduct'],       20);
    }

    public function install()
    {
        add_option('ucc_plugin_token', '');
    }

    /**
     * @return bool
     */
    private function isConfigured()
    {
        if (!empty(get_option('ucc_plugin_token'))) {
            return true;
        }

        return false;
    }

    /**
     * @param $uccPluginToken
     * @return bool
     */
    private function validatePluginToken($uccPluginToken)
    {
        $version = 'UNKNOWN';
        $plugins = get_plugins();
        foreach ($plugins as $key => $value) {
            if ($key == 'woocommerce/woocommerce.php') {
                $version = $value['Version'];
            }
        }

        $apiUrl = 'https://api.ucancommerce.com/api/v1/plugin/installation';
        $data = json_encode([
            'pluginToken' => strval($uccPluginToken),
            'shopPlatform' => 'WOOCOMMERCE',
            'platformVersion' => $version,
        ]);

        $headers = array(
            "Content-Type: application/json",
        );

        $curl = curl_init($apiUrl);
        curl_setopt($curl, CURLOPT_URL, $apiUrl);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);

        curl_exec($curl);

        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if ($code == 200) {
            return true;
        }

        return false;
    }

    public function pluginOptions()
    {
        if (count($_POST)) {
            if ($this->validatePluginToken($_POST['ucc_plugin_token'])) {
                update_option('ucc_plugin_token', $_POST['ucc_plugin_token']);
            } else {
                $uccErrors = new WP_Error('required', __('Code was not validated properly.'));
            }
        }

        include(plugin_dir_path(__DIR__). 'admin/settings_page.php');
    }

    public function hookWooCommerceInit() {
        if (!$this->isConfigured()) {
            return;
        }

        wp_enqueue_script('ucc-remote', 'https://app.ucancommerce.com/assets/scripts/ucc-pixel.js', [], null, true);
        wp_enqueue_script('ucc-bottom', plugin_dir_url(__DIR__) . 'public/js/ucc_bottom.js', [], false, true);
    }

    public function hookAfterSingleProduct()
    {
        if (!$this->isConfigured()) {
            return;
        }

        global $product;
        $uccProduct = [
            'id'    => get_post_meta($product->get_id(), '_custom_product_ucc_gtin', true),
            'price' => number_format($product->get_price(), 2),
        ];

        include(plugin_dir_path(__DIR__). 'woocommerce/woocommerce_after_single_product.php');

        wp_enqueue_script('ucc-product', plugin_dir_url(__DIR__) . 'public/js/ucc_product.js', [], false, true);
    }

    public function hookOrderReceived($order_id)
    {
        if (!$this->isConfigured()) {
            return;
        }

        $order = wc_get_order($order_id);
        $uccProducts = [];
        foreach ($order->get_items() as $product) {
            $tempProduct = $product->get_product();

            $uccProducts[] = [
                'id'    => get_post_meta($product->get_product_id(), '_custom_product_ucc_gtin', true),
                'qty'   => $product->get_quantity(),
                'price' => number_format($tempProduct->get_price(), 2),
            ];
        }

        include(plugin_dir_path(__DIR__) . 'woocommerce/woocommerce_thankyou.php');

        wp_enqueue_script('ucc-order-confirmation', plugin_dir_url(__DIR__) . 'public/js/ucc_order_confirmation.js', [], false, true);
    }

    public function hookProductInventory()
    {
        global $woocommerce, $post;
        echo '<div class="product_custom_field">';
        woocommerce_wp_text_input(
            array(
                'id' => '_custom_product_ucc_gtin',
                'placeholder' => '',
                'label' => __('GTIN code', 'woocommerce'),
                'desc_tip' => 'true'
            )
        );
        echo '</div>';
    }

    public function hookUpdateProduct($post_id)
    {
        $customProductUccGtin = $_POST['_custom_product_ucc_gtin'];
        if (!empty($customProductUccGtin))
            update_post_meta($post_id, '_custom_product_ucc_gtin', esc_attr($customProductUccGtin));
    }
}
